#!/usr/bin/env python

"""
Database utilities.

Copyright (C) 2016 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

import re

class DatabaseOperations:

    "Special database-related operations."

    def __init__(self, column_names=None, filter_values=None):
        self.column_names = column_names
        self.filter_values = filter_values

    def get_query(self, query, columns=None, values=None):

        """
        Return 'query' parameterised with condition clauses indicated by
        ":condition" in 'query' that are themselves populated using the given
        'columns' and 'values' together with any conditions provided when
        initialising this class.
        """

        columns = self.merge_default_columns(columns)
        values = self.merge_default_values(values)

        condition = self.get_condition(columns)

        # Replace ":condition", replicating the values the appropriate number of
        # times.

        query, count = re.subn(":condition(?=[^a-zA-Z]|$)", condition, query)
        all_values = values * count

        # Replace ":columns" and ":values", replicating the values again.

        columnlist = self.columnlist(columns)
        placeholders = self.placeholders(values)

        query, _count = re.subn(":columns(?=[^a-zA-Z]|$)", columnlist, query)
        query, count = re.subn(":values(?=[^a-zA-Z]|$)", placeholders, query)
        all_values += values * count

        return query, all_values

    def get_condition(self, columns=None):

        "Return a condition clause featuring the given 'columns'."

        l = []
        for column in columns:
            if " " in column:
                l.append(column)
            else:
                l.append("%s = ?" % column)

        return "where %s" % " and ".join(l)

    def merge_default_columns(self, columns=None):
        return list(self.column_names or []) + list(columns or [])

    def merge_default_values(self, values=None):
        return list(self.filter_values or []) + list(values or [])

    def columnlist(self, columns=None):
        return ", ".join(columns)

    def placeholders(self, values=None):
        return ", ".join(["?"] * len(values))

# vim: tabstop=4 expandtab shiftwidth=4
